@@ -57,15 +57,12 @@ OR PERFORMANCE OF THIS SOFTWARE.
 #include <errno.h>
 
 #include "utils.h"
-#include "scanner-utils.h"
 #include "table.h"
+#include "scanner-utils.h"
 #include "paths.h"
 #include "utf8.h"
 #include "parser.h"
 
-#define MAX_LHS_LEN 10
-#define MAX_INCLUDE_DEPTH 5
-
 /*
  * Grammar adapted from libX11/modules/im/ximcp/imLcPrs.c.
  * See also the XCompose(5) manpage.
@@ -174,6 +171,7 @@ skip_more_whitespace_and_comments:
         while (!scanner_eof(s) && !scanner_eol(s) && scanner_peek(s) != '\"') {
             if (scanner_chr(s, '\\')) {
                 uint8_t o;
+                size_t start_pos = s->pos;
                 if (scanner_chr(s, '\\')) {
                     scanner_buf_append(s, '\\');
                 }
@@ -181,16 +179,30 @@ skip_more_whitespace_and_comments:
                     scanner_buf_append(s, '"');
                 }
                 else if (scanner_chr(s, 'x') || scanner_chr(s, 'X')) {
-                    if (scanner_hex(s, &o))
+                    if (scanner_hex(s, &o) && is_valid_char((char) o)) {
                         scanner_buf_append(s, (char) o);
-                    else
-                        scanner_warn(s, "illegal hexadecimal escape sequence in string literal");
+                    } else {
+                        scanner_warn_with_code(s,
+                            XKB_WARNING_INVALID_ESCAPE_SEQUENCE,
+                            "illegal hexadecimal escape sequence (%.*s) in string literal",
+                            (int) (s->pos - start_pos + 1), &s->s[start_pos - 1]);
+                    }
                 }
-                else if (scanner_oct(s, &o)) {
+                else if (scanner_oct(s, &o) && is_valid_char((char) o)) {
                     scanner_buf_append(s, (char) o);
                 }
+                else if (s->pos > start_pos) {
+                    scanner_warn_with_code(s,
+                        XKB_WARNING_INVALID_ESCAPE_SEQUENCE,
+                        "illegal octal escape sequence (%.*s) in string literal",
+                        (int) (s->pos - start_pos + 1), &s->s[start_pos - 1]);
+                    /* Ignore. */
+                }
                 else {
-                    scanner_warn(s, "unknown escape sequence (%c) in string literal", scanner_peek(s));
+                    scanner_warn_with_code(s,
+                        XKB_WARNING_UNKNOWN_CHAR_ESCAPE_SEQUENCE,
+                        "unknown escape sequence (\\%c) in string literal",
+                        scanner_peek(s));
                     /* Ignore. */
                 }
             } else {
@@ -232,10 +244,9 @@ skip_more_whitespace_and_comments:
         return TOK_IDENT;
     }
 
+    scanner_err(s, "unrecognized token");
     /* Discard rest of line. */
     scanner_skip_to_eol(s);
-
-    scanner_err(s, "unrecognized token");
     return TOK_ERROR;
 }
 
@@ -317,7 +328,7 @@ struct production {
     xkb_keysym_t lhs[MAX_LHS_LEN];
     unsigned int len;
     xkb_keysym_t keysym;
-    char string[256];
+    char string[XKB_COMPOSE_MAX_STRING_SIZE];
     /* At least one of these is true. */
     bool has_keysym;
     bool has_string;
@@ -332,8 +343,8 @@ add_production(struct xkb_compose_table *table, struct scanner *s,
                const struct production *production)
 {
     unsigned lhs_pos = 0;
-    uint16_t curr = darray_size(table->nodes) == 1 ? 0 : 1;
-    uint16_t *pptr = NULL;
+    uint32_t curr = darray_size(table->nodes) == 1 ? 0 : 1;
+    uint32_t *pptr = NULL;
     struct compose_node *node = NULL;
 
     /* Warn before potentially going over the limit, discard silently after. */
@@ -457,6 +468,24 @@ resolve_modifier(const char *name)
     return XKB_MOD_INVALID;
 }
 
+/* Parse a string literal ("...") and return the corresponding unescaped string,
+ * or NULL if it fails.
+ * This is aimed only for testing (un)escaping characters. */
+char *
+parse_string_literal(struct xkb_context *ctx, const char *string)
+{
+    struct scanner s;
+    union lvalue val;
+    scanner_init(&s, ctx, string, strlen(string), "(unamed)", NULL);
+    switch (lex(&s, &val)) {
+        case TOK_STRING:
+            return strdup(val.string.str);
+        default:
+            fprintf(stderr, "ERROR: %s\n", s.s);
+            return NULL;
+    }
+}
+
 static bool
 parse(struct xkb_compose_table *table, struct scanner *s,
       unsigned include_depth);
@@ -515,6 +544,16 @@ parse(struct xkb_compose_table *table, struct scanner *s,
     enum { MAX_ERRORS = 10 };
     int num_errors = 0;
 
+    /* Basic detection of wrong character encoding.
+       The first character relevant to the grammar must be ASCII:
+       whitespace, include, modifier list, keysym, comment */
+    if (!scanner_check_supported_char_encoding(s)) {
+        scanner_err(s,
+                    "This could be a file encoding issue. "
+                    "Supported file encodings are ASCII and UTF-8.");
+        goto fail;
+    }
+
 initial:
     production.len = 0;
     production.has_keysym = false;
@@ -648,8 +687,10 @@ rhs:
             scanner_warn(s, "right-hand side string must not be empty; skipping line");
             goto skip;
         }
-        if (val.string.len >= sizeof(production.string)) {
-            scanner_warn(s, "right-hand side string is too long; skipping line");
+        if (val.string.len > sizeof(production.string)) {
+            scanner_warn(s,
+                         "right-hand side string is too long: expected max: %d, got: %d; "
+                         "skipping line", (int)sizeof(production.string) - 1, (int)val.string.len);
             goto skip;
         }
         strcpy(production.string, val.string.str);
@@ -727,7 +768,9 @@ parse_file(struct xkb_compose_table *table, FILE *file, const char *file_name)
 
     ok = map_file(file, &string, &size);
     if (!ok) {
-        log_err(table->ctx, "Couldn't read Compose file %s: %s\n",
+        log_err(table->ctx,
+                XKB_LOG_MESSAGE_NO_ID,
+                "Couldn't read Compose file %s: %s\n",
                 file_name, strerror(errno));
         return false;
     }
