@@ -34,6 +34,7 @@
 #include "xkbcommon/xkbregistry.h"
 #include "utils.h"
 #include "util-list.h"
+#include "util-mem.h"
 
 struct rxkb_object;
 
@@ -564,18 +565,14 @@ rxkb_context_include_path_append(struct rxkb_context *ctx, const char *path)
         return false;
     }
 
-    tmp = strdup(path);
-    if (!tmp)
-        goto err;
-
     err = stat(path, &stat_buf);
     if (err != 0)
-        goto err;
+        return false;
     if (!S_ISDIR(stat_buf.st_mode))
-        goto err;
+        return false;
 
     if (!check_eaccess(path, R_OK | X_OK))
-        goto err;
+        return false;
 
     /* Pre-filter for the 99.9% case - if we can't assemble the default ruleset
      * path, complain here instead of during parsing later. The niche cases
@@ -583,22 +580,22 @@ rxkb_context_include_path_append(struct rxkb_context *ctx, const char *path)
      */
     if (!snprintf_safe(rules, sizeof(rules), "%s/rules/%s.xml",
                        path, DEFAULT_XKB_RULES))
-        goto err;
+        return false;
+
+    tmp = strdup(path);
+    if (!tmp)
+        return false;
 
     darray_append(ctx->includes, tmp);
 
     return true;
-
-err:
-    free(tmp);
-    return false;
 }
 
 XKB_EXPORT bool
 rxkb_context_include_path_append_default(struct rxkb_context *ctx)
 {
     const char *home, *xdg, *root, *extra;
-    char *user_path;
+    char user_path[PATH_MAX];
     bool ret = false;
 
     if (ctx->context_state != CONTEXT_NEW) {
@@ -610,26 +607,17 @@ rxkb_context_include_path_append_default(struct rxkb_context *ctx)
 
     xdg = rxkb_context_getenv(ctx, "XDG_CONFIG_HOME");
     if (xdg != NULL) {
-        user_path = asprintf_safe("%s/xkb", xdg);
-        if (user_path) {
+        if (snprintf_safe(user_path, sizeof(user_path), "%s/xkb", xdg))
             ret |= rxkb_context_include_path_append(ctx, user_path);
-            free(user_path);
-        }
     } else if (home != NULL) {
         /* XDG_CONFIG_HOME fallback is $HOME/.config/ */
-        user_path = asprintf_safe("%s/.config/xkb", home);
-        if (user_path) {
+        if (snprintf_safe(user_path, sizeof(user_path), "%s/.config/xkb", home))
             ret |= rxkb_context_include_path_append(ctx, user_path);
-            free(user_path);
-        }
     }
 
     if (home != NULL) {
-        user_path = asprintf_safe("%s/.xkb", home);
-        if (user_path) {
+        if (snprintf_safe(user_path, sizeof(user_path), "%s/.xkb", home))
             ret |= rxkb_context_include_path_append(ctx, user_path);
-            free(user_path);
-        }
     }
 
     extra = rxkb_context_getenv(ctx, "XKB_CONFIG_EXTRA_PATH");
@@ -721,44 +709,73 @@ extract_text(xmlNode *node)
     return NULL;
 }
 
+/* Data from “configItem” node */
+struct config_item {
+    char *name;
+    char *description;
+    char *brief;
+    char *vendor;
+    enum rxkb_popularity popularity;
+};
+
+#define config_item_new(popularity_) { \
+    .name = NULL, \
+    .description = NULL, \
+    .brief = NULL, \
+    .vendor = NULL, \
+    .popularity = popularity_ \
+}
+
+static void
+config_item_free(struct config_item *config) {
+    free(config->name);
+    free(config->description);
+    free(config->brief);
+    free(config->vendor);
+}
+
 static bool
-parse_config_item(struct rxkb_context *ctx,
-                  xmlNode *parent,
-                  char **name,
-                  char **description,
-                  char **brief,
-                  char **vendor)
+parse_config_item(struct rxkb_context *ctx, xmlNode *parent,
+                  struct config_item *config)
 {
     xmlNode *node = NULL;
     xmlNode *ci = NULL;
 
     for (ci = parent->children; ci; ci = ci->next) {
         if (is_node(ci, "configItem")) {
-            *name = NULL;
-            *description = NULL;
-            *brief = NULL;
-            *vendor = NULL;
+            /* Process attributes */
+            xmlChar *raw_popularity = xmlGetProp(ci, (const xmlChar*)"popularity");
+            if (raw_popularity) {
+                if (xmlStrEqual(raw_popularity, (const xmlChar*)"standard"))
+                    config->popularity = RXKB_POPULARITY_STANDARD;
+                else if (xmlStrEqual(raw_popularity, (const xmlChar*)"exotic"))
+                    config->popularity = RXKB_POPULARITY_EXOTIC;
+                else
+                    log_err(ctx,
+                            "xml:%d: invalid popularity attribute: expected "
+                            "'standard' or 'exotic', got: '%s'\n",
+                            ci->line, raw_popularity);
+            }
+            xmlFree(raw_popularity);
 
+            /* Process children */
             for (node = ci->children; node; node = node->next) {
                 if (is_node(node, "name"))
-                    *name = extract_text(node);
+                    config->name = extract_text(node);
                 else if (is_node(node, "description"))
-                    *description = extract_text(node);
+                    config->description = extract_text(node);
                 else if (is_node(node, "shortDescription"))
-                    *brief = extract_text(node);
+                    config->brief = extract_text(node);
                 else if (is_node(node, "vendor"))
-                    *vendor = extract_text(node);
+                    config->vendor = extract_text(node);
                 /* Note: the DTD allows for vendor + brief but models only use
                  * vendor and everything else only uses shortDescription */
             }
 
-            if (!*name || !strlen(*name))  {
+            if (!config->name || !strlen(config->name))  {
                 log_err(ctx, "xml:%d: missing required element 'name'\n",
                         ci->line);
-                free(*name);
-                free(*description);
-                free(*brief);
-                free(*vendor);
+                config_item_free(config);
                 return false;
             }
 
@@ -773,34 +790,31 @@ static void
 parse_model(struct rxkb_context *ctx, xmlNode *model,
             enum rxkb_popularity popularity)
 {
-    char *name, *description, *brief, *vendor;
+    struct config_item config = config_item_new(popularity);
 
-    if (parse_config_item(ctx, model, &name, &description, &brief, &vendor)) {
+    if (parse_config_item(ctx, model, &config)) {
         struct rxkb_model *m;
 
         list_for_each(m, &ctx->models, base.link) {
-            if (streq(m->name, name)) {
-                free(name);
-                free(description);
-                free(brief);
-                free(vendor);
+            if (streq(m->name, config.name)) {
+                config_item_free(&config);
                 return;
             }
         }
 
         /* new model */
         m = rxkb_model_create(&ctx->base);
-        m->name = name;
-        m->description = description;
-        m->vendor = vendor;
-        m->popularity = popularity;
+        m->name = steal(&config.name);
+        m->description = steal(&config.description);
+        m->vendor = steal(&config.vendor);
+        m->popularity = config.popularity;
         list_append(&ctx->models, &m->base.link);
     }
 }
 
 static void
 parse_model_list(struct rxkb_context *ctx, xmlNode *model_list,
-                enum rxkb_popularity popularity)
+                 enum rxkb_popularity popularity)
 {
     xmlNode *node = NULL;
 
@@ -863,14 +877,14 @@ parse_variant(struct rxkb_context *ctx, struct rxkb_layout *l,
               xmlNode *variant, enum rxkb_popularity popularity)
 {
     xmlNode *ci;
-    char *name, *description, *brief, *vendor;
+    struct config_item config = config_item_new(popularity);
 
-    if (parse_config_item(ctx, variant, &name, &description, &brief, &vendor)) {
+    if (parse_config_item(ctx, variant, &config)) {
         struct rxkb_layout *v;
         bool exists = false;
 
         list_for_each(v, &ctx->layouts, base.link) {
-            if (streq(v->name, name) && streq(v->name, l->name)) {
+            if (streq(v->name, config.name) && streq(v->name, l->name)) {
                 exists = true;
                 break;
             }
@@ -881,11 +895,11 @@ parse_variant(struct rxkb_context *ctx, struct rxkb_layout *l,
             list_init(&v->iso639s);
             list_init(&v->iso3166s);
             v->name = strdup(l->name);
-            v->variant = name;
-            v->description = description;
+            v->variant = steal(&config.name);
+            v->description = steal(&config.description);
             // if variant omits brief, inherit from parent layout.
-            v->brief = brief == NULL ? strdup_safe(l->brief) : brief;
-            v->popularity = popularity;
+            v->brief = config.brief == NULL ? strdup_safe(l->brief) : steal(&config.brief);
+            v->popularity = config.popularity;
             list_append(&ctx->layouts, &v->base.link);
 
             for (ci = variant->children; ci; ci = ci->next) {
@@ -926,10 +940,7 @@ parse_variant(struct rxkb_context *ctx, struct rxkb_layout *l,
                 }
             }
         } else {
-            free(name);
-            free(description);
-            free(brief);
-            free(vendor);
+            config_item_free(&config);
         }
     }
 }
@@ -950,16 +961,16 @@ static void
 parse_layout(struct rxkb_context *ctx, xmlNode *layout,
              enum rxkb_popularity popularity)
 {
-    char *name, *description, *brief, *vendor;
+    struct config_item config = config_item_new(popularity);
     struct rxkb_layout *l;
     xmlNode *node = NULL;
     bool exists = false;
 
-    if (!parse_config_item(ctx, layout, &name, &description, &brief, &vendor))
+    if (!parse_config_item(ctx, layout, &config))
         return;
 
     list_for_each(l, &ctx->layouts, base.link) {
-        if (streq(l->name, name) && l->variant == NULL) {
+        if (streq(l->name, config.name) && l->variant == NULL) {
             exists = true;
             break;
         }
@@ -969,17 +980,14 @@ parse_layout(struct rxkb_context *ctx, xmlNode *layout,
         l = rxkb_layout_create(&ctx->base);
         list_init(&l->iso639s);
         list_init(&l->iso3166s);
-        l->name = name;
+        l->name = steal(&config.name);
         l->variant = NULL;
-        l->description = description;
-        l->brief = brief;
-        l->popularity = popularity;
+        l->description = steal(&config.description);
+        l->brief = steal(&config.brief);
+        l->popularity = config.popularity;
         list_append(&ctx->layouts, &l->base.link);
     } else {
-        free(name);
-        free(description);
-        free(brief);
-        free(vendor);
+        config_item_free(&config);
     }
 
     for (node = layout->children; node; node = node->next) {
@@ -1014,25 +1022,22 @@ static void
 parse_option(struct rxkb_context *ctx, struct rxkb_option_group *group,
              xmlNode *option, enum rxkb_popularity popularity)
 {
-    char *name, *description, *brief, *vendor;
+    struct config_item config = config_item_new(popularity);
 
-    if (parse_config_item(ctx, option, &name, &description, &brief, &vendor)) {
+    if (parse_config_item(ctx, option, &config)) {
         struct rxkb_option *o;
 
         list_for_each(o, &group->options, base.link) {
-            if (streq(o->name, name)) {
-                free(name);
-                free(description);
-                free(brief);
-                free(vendor);
+            if (streq(o->name, config.name)) {
+                config_item_free(&config);
                 return;
             }
         }
 
         o = rxkb_option_create(&group->base);
-        o->name = name;
-        o->description = description;
-        o->popularity = popularity;
+        o->name = steal(&config.name);
+        o->description = steal(&config.description);
+        o->popularity = config.popularity;
         list_append(&group->options, &o->base.link);
     }
 }
@@ -1041,17 +1046,17 @@ static void
 parse_group(struct rxkb_context *ctx, xmlNode *group,
             enum rxkb_popularity popularity)
 {
-    char *name, *description, *brief, *vendor;
+    struct config_item config = config_item_new(popularity);
     struct rxkb_option_group *g;
     xmlNode *node = NULL;
     xmlChar *multiple;
     bool exists = false;
 
-    if (!parse_config_item(ctx, group, &name, &description, &brief, &vendor))
+    if (!parse_config_item(ctx, group, &config))
         return;
 
     list_for_each(g, &ctx->option_groups, base.link) {
-        if (streq(g->name, name)) {
+        if (streq(g->name, config.name)) {
             exists = true;
             break;
         }
@@ -1059,9 +1064,9 @@ parse_group(struct rxkb_context *ctx, xmlNode *group,
 
     if (!exists) {
         g = rxkb_option_group_create(&ctx->base);
-        g->name = name;
-        g->description = description;
-        g->popularity = popularity;
+        g->name = steal(&config.name);
+        g->description = steal(&config.description);
+        g->popularity = config.popularity;
 
         multiple = xmlGetProp(group, (const xmlChar*)"allowMultipleSelection");
         if (multiple && xmlStrEqual(multiple, (const xmlChar*)"true"))
@@ -1071,10 +1076,7 @@ parse_group(struct rxkb_context *ctx, xmlNode *group,
         list_init(&g->options);
         list_append(&ctx->option_groups, &g->base.link);
     } else {
-        free(name);
-        free(description);
-        free(brief);
-        free(vendor);
+        config_item_free(&config);
     }
 
     for (node = group->children; node; node = node->next) {
@@ -1159,9 +1161,12 @@ validate(struct rxkb_context *ctx, xmlDoc *doc)
     xmlValidCtxt *dtdvalid = NULL;
     xmlDtd *dtd = NULL;
     xmlParserInputBufferPtr buf = NULL;
-    /* This is a modified version of the xkeyboard-config xkb.dtd. That one
-     * requires modelList, layoutList and optionList, we
-     * allow for any of those to be missing.
+    /* This is a modified version of the xkeyboard-config xkb.dtd:
+     * • xkeyboard-config requires modelList, layoutList and optionList,
+     *   but we allow for any of those to be missing.
+     * • xkeyboard-config sets default value of “popularity” to “standard”,
+     *   but we set this value depending if we are currently parsing an
+     *   “extras” rule file.
      */
     const char dtdstr[] =
         "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
@@ -1178,7 +1183,7 @@ validate(struct rxkb_context *ctx, xmlDoc *doc)
         "<!ATTLIST group allowMultipleSelection (true|false) \"false\">\n"
         "<!ELEMENT option (configItem)>\n"
         "<!ELEMENT configItem (name, shortDescription?, description?, vendor?, countryList?, languageList?, hwList?)>\n"
-        "<!ATTLIST configItem popularity (standard|exotic) \"standard\">\n"
+        "<!ATTLIST configItem popularity (standard|exotic) #IMPLIED>\n"
         "<!ELEMENT name (#PCDATA)>\n"
         "<!ELEMENT shortDescription (#PCDATA)>\n"
         "<!ELEMENT description (#PCDATA)>\n"
