@@ -21,12 +21,16 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include "evdev-scancodes.h"
 #include "test.h"
 
 int
 main(void)
 {
+    test_init();
+
     struct xkb_context *ctx = test_get_context(0);
     struct xkb_keymap *keymap;
 
@@ -297,7 +301,7 @@ main(void)
                         KEY_RIGHTALT,    DOWN,  XKB_KEY_ISO_Level5_Shift,  NEXT,
                         /* XXX: xkeyboard-config is borked when de(neo) is
                          *      not the first group - not our fault. We test
-                         *      Level5 seprately below with only de(neo). */
+                         *      Level5 separately below with only de(neo). */
                         /* KEY_5,           BOTH,  XKB_KEY_periodcentered,    NEXT, */
                         /* KEY_E,           BOTH,  XKB_KEY_Up,                NEXT, */
                         /* KEY_SPACE,       BOTH,  XKB_KEY_KP_0,              NEXT, */
@@ -318,7 +322,7 @@ main(void)
                         KEY_SPACE,       BOTH,  XKB_KEY_KP_0,              NEXT,
                         KEY_KP8,         BOTH,  XKB_KEY_KP_Up,             NEXT,
                         KEY_ESC,         BOTH,  XKB_KEY_Escape,            NEXT,
-                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Shift,  NEXT,
+                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Lock,   NEXT,
 
                         /* Level 6. */
                         KEY_RIGHTALT,    DOWN,  XKB_KEY_ISO_Level5_Shift,  NEXT,
@@ -330,7 +334,7 @@ main(void)
                         KEY_KP8,         BOTH,  XKB_KEY_KP_Up,             NEXT,
                         KEY_ESC,         BOTH,  XKB_KEY_Escape,            NEXT,
                         KEY_RIGHTSHIFT,  UP,    XKB_KEY_Caps_Lock,         NEXT,
-                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Shift,  NEXT,
+                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Lock,   NEXT,
 
                         /* Level 7. */
                         KEY_RIGHTALT,    DOWN,  XKB_KEY_ISO_Level5_Shift,  NEXT,
@@ -341,19 +345,22 @@ main(void)
                         KEY_KP8,         BOTH,  KS("U22C2"),               NEXT,
                         KEY_ESC,         BOTH,  XKB_KEY_Escape,            NEXT,
                         KEY_CAPSLOCK,    UP,    XKB_KEY_ISO_Level3_Shift,  NEXT,
-                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Shift,  NEXT,
+                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Lock,   NEXT,
 
                         /* Level 8. */
                         KEY_RIGHTALT,    DOWN,  XKB_KEY_ISO_Level5_Shift,  NEXT,
                         KEY_CAPSLOCK,    DOWN,  XKB_KEY_ISO_Level3_Shift,  NEXT,
                         KEY_RIGHTSHIFT,  DOWN,  XKB_KEY_Shift_R,           NEXT,
-                        /* This doesn't actually lock Level5. Not our fault. */
                         KEY_TAB,         BOTH,  XKB_KEY_ISO_Level5_Lock,   NEXT,
+                        KEY_V,           BOTH,  XKB_KEY_Greek_pi,          NEXT,
                         KEY_RIGHTSHIFT,  UP,    XKB_KEY_Caps_Lock,         NEXT,
+                        KEY_V,           BOTH,  XKB_KEY_asciitilde,        NEXT,
                         KEY_CAPSLOCK,    UP,    XKB_KEY_ISO_Level3_Shift,  NEXT,
-                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Shift,  NEXT,
+                        KEY_V,           BOTH,  XKB_KEY_p,                 NEXT,
+                        KEY_RIGHTALT,    UP,    XKB_KEY_ISO_Level5_Lock,   NEXT,
+                        /* Locks Level 5. */
 
-                        KEY_V,           BOTH,  XKB_KEY_p,                 FINISH));
+                        KEY_V,           BOTH,  XKB_KEY_Return,            FINISH));
 
 
     xkb_keymap_unref(keymap);
