@@ -37,6 +37,9 @@ usage(void)
            "Global options:\n"
            "  -h, --help ...... show this help and exit\n"
            "  -V, --version ... show version information and exit\n"
+           /* WARNING: The following is parsed by the bash completion script.
+            *          Any change to the format (in particular to the indentation)
+            *          should kept in the script in sync. */
            "Commands:\n"
 #if HAVE_XKBCLI_LIST
            "  list\n"
@@ -63,6 +66,11 @@ usage(void)
            "    Compile an XKB keymap\n"
            "\n"
 #endif
+#if HAVE_XKBCLI_COMPILE_COMPOSE
+           "  compile-compose\n"
+           "    Compile a Compose file\n"
+           "\n"
+#endif
 #if HAVE_XKBCLI_HOW_TO_TYPE
            "  how-to-type\n"
            "    Print key sequences to type a Unicode codepoint\n"
